/*
 * input.c: Filter class for the Atmolight-plugin
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <stdlib.h>
#include <vdr/tools.h>
#include <vdr/thread.h>
#include "math.h"

#include "defs.h"
#include "setup.h"
#include "filter.h"

// TODO: filterintern
tColorPacket filter_input, filter_output; // input and output of the filter

// --- cAtmoFilter-----------------------------------------------------------

cAtmoFilter::cAtmoFilter()
{
  // Initialization
  MeanFilter(true);
  PercentFilter(true);
}

cAtmoFilter::~cAtmoFilter()
{
}

tColorPacketInt cAtmoFilter::Filtering(tColorPacketInt ColorPacketInt)
{
  for (int i = 0; i < 5; i++) // TODO: im Filter mit unsigned int rechnen
  {
    filter_input.channel[i].r = (double) ColorPacketInt.channel[i].r / 255.0f;
    filter_input.channel[i].g = (double) ColorPacketInt.channel[i].g / 255.0f;
    filter_input.channel[i].b = (double) ColorPacketInt.channel[i].b / 255.0f;
  }

  // apply filter
  switch (AtmoSetup.FilterMode) // TODO: Namen
  {
    case 0:
         MeanFilter(false);
    break;

    case 1:
         PercentFilter(false);
    break;

    default:
         filter_output = filter_input;
    break;
  }

  for (int i = 0; i < 5; i++) // TODO: im Filter mit unsigned int rechnen
  {
    filter_outputInt.channel[i].r = (unsigned int) (255.0f * filter_output.channel[i].r);
    filter_outputInt.channel[i].g = (unsigned int) (255.0f * filter_output.channel[i].g);
    filter_outputInt.channel[i].b = (unsigned int) (255.0f * filter_output.channel[i].b);
  }

  return filter_outputInt;
}

void cAtmoFilter::PercentFilter(bool init)
{
  static tColorPacket filter_output_old; // needed for the percentage filter

  if (init) // Initialization
  {
    memset(&filter_output_old, 0, sizeof(filter_output_old));
    return;
  }

  for (int ch = 0; ch < 5; ch++)
  {
    filter_output.channel[ch].r = ((filter_input.channel[ch].r * (100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].r *((double)AtmoSetup.PercentNew))) / 100.0f;
    filter_output.channel[ch].g = ((filter_input.channel[ch].g * (100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].g *((double)AtmoSetup.PercentNew))) / 100.0f;
    filter_output.channel[ch].b = ((filter_input.channel[ch].b * (100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].b *((double)AtmoSetup.PercentNew))) / 100.0f;
  }
  filter_output_old = filter_output;
}

void cAtmoFilter::MeanFilter(bool init)
{
  static tColorPacket mean_sums;   // needed vor the running mean value filter
  static tColorPacket mean_values;
  static tColorPacket filter_output_old; // needed for the percentage filter
  static int filter_length_old;
  char reinitialize = 0;

  if (init) // Initialization
  {
    memset(&filter_output_old, 0, sizeof(filter_output_old));
    memset(&mean_sums, 0, sizeof(mean_sums));
    memset(&mean_values, 0, sizeof(mean_values));
    return;
  }

  if (filter_length_old != AtmoSetup.FilterMeanCount) // if filter_length has changed
  {
    reinitialize = 1; // force reinitialization of the filter
  }
  filter_length_old = AtmoSetup.FilterMeanCount;

  for (int ch = 0; ch < 5; ch++)
  {
    // calculate the mean-value filters
    mean_sums.channel[ch].r += (filter_input.channel[ch].r-mean_values.channel[ch].r);   // red
    mean_values.channel[ch].r = mean_sums.channel[ch].r/((double)(filter_length_old)/20.0f);

    mean_sums.channel[ch].g += (filter_input.channel[ch].g-mean_values.channel[ch].g);   // green
    mean_values.channel[ch].g = mean_sums.channel[ch].g/((double)(filter_length_old)/20.0f);

    mean_sums.channel[ch].b += (filter_input.channel[ch].b-mean_values.channel[ch].b);   // blue
    mean_values.channel[ch].b = mean_sums.channel[ch].b/((double)(filter_length_old)/20.0f);

    // check, if there is a jump -> check if differences between actual values and filter values are too big

    double dist;  // distance between the two colors in the 3D RGB space
    dist = (mean_values.channel[ch].r - filter_input.channel[ch].r)*(mean_values.channel[ch].r - filter_input.channel[ch].r)+ \
           (mean_values.channel[ch].g - filter_input.channel[ch].g)*(mean_values.channel[ch].g - filter_input.channel[ch].g)+ \
           (mean_values.channel[ch].b - filter_input.channel[ch].b)*(mean_values.channel[ch].b - filter_input.channel[ch].b);

    if (dist > 0) { dist = sqrt(dist); } // avoid sqrt(0) !

    // compare calculated distance with the filter threshold
    if ( (dist > (double)AtmoSetup.FilterMeanThreshold*0.0141f) || ( reinitialize == 1) )
    {
      // dsyslog("atmolight: filter jump detected!!");

      // if this occurs, set the long filters to the result of the short filters
      mean_values.channel[ch].r = filter_input.channel[ch].r;
      mean_sums.channel[ch].r = filter_input.channel[ch].r*((double)(filter_length_old)/20.0f);

      mean_values.channel[ch].g = filter_input.channel[ch].g;
      mean_sums.channel[ch].g = filter_input.channel[ch].g*((double)(filter_length_old)/20.0f);

      mean_values.channel[ch].b = filter_input.channel[ch].b;
      mean_sums.channel[ch].b = filter_input.channel[ch].b*((double)(filter_length_old)/20.0f);
    }
    else
    {
      // apply an additional percent filter and return calculated values
      filter_output.channel[ch].r = ((mean_values.channel[ch].r * (double)(100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].r *((double)AtmoSetup.PercentNew))) / 100.0f;
      filter_output.channel[ch].g = ((mean_values.channel[ch].g * (double)(100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].g *((double)AtmoSetup.PercentNew))) / 100.0f;
      filter_output.channel[ch].b = ((mean_values.channel[ch].b * (double)(100.0f-(double)AtmoSetup.PercentNew)) + (filter_output_old.channel[ch].b *((double)AtmoSetup.PercentNew))) / 100.0f;
    }  
  }
  filter_output_old = filter_output;
}
