/*
 * setup.h: The setup for the Atmolight-plugin
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#ifndef _ATMO_SETUP__H
#define _ATMO_SETUP__H

#include <vdr/plugin.h>

typedef enum
{
  black = 0,
  white,
  red,
  green,
  blue,
  yellow,
  cyan,
  magenta
} tDefaultColors;

typedef enum
{
  no_pos = 0,
  all,
  center,
  left,
  right,
  top,
  bottom,
} tPosition;

typedef enum
{
  live_picture_hsv = 0,
  default_color,
  static_color,
  white_calibration,
} tMode;

typedef enum
{
  no_input = 0,
  video0
} tInput;

typedef enum
{
  no_output = 0,
  serial,
  network
} tOutput;

// --- cAtmoSetup -------------------------------------------------------------

class cAtmoSetup {
private:
public:
  char *output_arg; // for sending data
  tInput  AtmoInput;  // input device
  tOutput AtmoOutput; // output device

  // internal status
  bool atmo_on; // Atmolight on/off

  // saved parameters
  int StartMode;    // Atmoligt start mode: on, off, timer dependent
  int PercentNew;   // percentage of new color (for filtering)
  int ValOffset;    // value offset
  int GammaMode;    // gamma Mode 0=uniform 1=by channel
  int GammaUni;     // gamma correction for all channels
  int GammaRed;     // gamma correction for red
  int GammaGreen;   // gamma correction for green
  int GammaBlue;    // gamma correction for blue
  int DarkLimit;    // HSV.V < limit -> HSV.V=0 (no light)
  tMode Mode;       // atmolight-mode
  tDefaultColors DefaultColor;
  int StaticRed_center;       // static mode: red center channel
  int StaticGreen_center;     // static mode: green center channel
  int StaticBlue_center;      // static mode: blue center channel
  int StaticRed_left;         // static mode: red left channel
  int StaticGreen_left;       // static mode: green left channel
  int StaticBlue_left;        // static mode: blue left channel
  int StaticRed_right;        // static mode: red right channel
  int StaticGreen_right;      // static mode: green right channel
  int StaticBlue_right;       // static mode: blue right channel
  int StaticRed_top;          // static mode: red top channel
  int StaticGreen_top;        // static mode: green top channel
  int StaticBlue_top;         // static mode: blue top channel
  int StaticRed_bottom;       // static mode: red bottom channel
  int StaticGreen_bottom;     // static mode: green bottom channel
  int StaticBlue_bottom;      // static mode: blue bottom channel
  int CalibRed;     // calibration mode: red
  int CalibGreen;   // calibration mode: green
  int CalibBlue;    // calibration mode: blue
  int WeightPow;
  int HueWinSize;
  int SatWinSize;
  int ValMode;
  int FilterMode;
  int FilterMeanCount;
  int FilterMeanThreshold;
  int WidescreenMode; // if set to 1, top and bottom of the pic will be cut before calculation (4:3 pic viewed as 16:9)

  cAtmoSetup();
  bool SetupParse(const char *Name, const char *Value);
  void GetStartMode(void);
  void SetDefault(tDefaultColors col);
  void SetStatic(tPosition pos, int r, int g, int b);
};
extern cAtmoSetup AtmoSetup;

class cMenuSetupAtmo : public cMenuSetupPage {
private:
  void Set(void);
protected:
  virtual void Store(void);
  virtual eOSState ProcessKey(eKeys Key);
public:
  cMenuSetupAtmo();
};

#endif // _ATMO_SETUP__H
